#!/usr/bin/env python
# encoding: utf-8
#
# ** header v3.0
# This file is a part of the CaosDB Project.
#
# Copyright (C) 2019 Henrik tom Wörden
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# ** end header
import os
import unittest
from copy import deepcopy
from tempfile import NamedTemporaryFile

import caosdb as db
from caosadvancedtools.cache import Cache


class CacheTest(unittest.TestCase):
    def setUp(self):
        self.cache = Cache(db_file=NamedTemporaryFile(delete=False).name)
        self.cache.create_cache()

    def test_hash(self):
        ent = db.Record()
        assert type(Cache.hash_entity(ent)) is str
        assert (Cache.hash_entity(ent) !=
                Cache.hash_entity(db.Record().add_parent("lol")))

    def test_insert(self):
        ent = db.Record()
        ent2 = db.Record()
        ent2.add_parent(name="Experiment")
        ent_hash = Cache.hash_entity(ent)
        print(ent_hash)
        ent2_hash = Cache.hash_entity(ent2)
        print(ent2_hash)
        self.cache.insert(ent2_hash, 1235)
        assert type(self.cache.check_existing(ent2_hash)) is int
        assert self.cache.check_existing(ent_hash) is None

    def test_hirarchy(self):
        assert isinstance(db.Record(), db.Entity)

    def tearDown(self):
        os.remove(self.cache.db_file)
